
/***************************************************
* By CCSC compiler
*  File Name: 20180409_k324_MK305B_1705.c
 * rev 1.0
 * rev 2.0 zero delay to turn relay off
 * rev 3.0 modify pot X*X, off delay=0to240
 * rev 3.1 turn off relay at initialization
 * rev 4.0 erase on delay
*  Description: get sound from microphone and turn on relay
*  RC0 in opemp in-, microphone
*  RC1 in opamp in+
*  RC2 out opamp out
*  RC3 out sound level led low
*  RC4 out sound level led mid
*  RC5 out sound level led high
*  RA0 in ADC, potentiometer
*  RA1 in ADC, sound
*  RA2 in na
*  RA3 in na
*  RA4 in na
*  RA5 out relay, active h
*  COPYRIGHT 2018 MYCOMKITS.COM, owned by CNET LIMITED
* $B$3$N%W%m%0%i%`$NCx:n8"$O!"@=:n<T!V%^%$%3%s%-%C%H%I%C%H%3%`1?1D!!M-8B2q<R%/%M%C%H!W$K5"B0$7$^$9!#(B
* $B2f!9$OCx:n8"$rJ|4~$7$F$$$^$;$s$,!"Ev%W%m%0%i%`$r;H$C$?3X=,$NCf$G%W%m%0%i%`$r<+M3$KJQ99$7$F$*;H$$$/$@$5$$!#(B
* The copyright of this program belongs to the producer "MYCOMKITS.COM management company CNET LIMITED". 
* We do not abandon copyright, but you can use and change the program in learning using this program. 
***************************************************/
//
#include <16F1705.h>
#DEVICE ADC=8
#fuses	INTRC_IO, NOWDT, NOMCLR, NOBROWNOUT, PUT, NOPROTECT, NODEBUG
#fuses	NOCLKOUT, NOWRT
#use	delay(CLOCK = 4000000)
//
#use   fast_io(a)  //a5 to 0
#use   fast_io(c)  //C5 to 0
//
#define Amode 0xDF         //port A all input, except A5
#define Cmode 0x07         //port C all output except RC0,1
#define LED1_ON output_low(PIN_C3)
#define LED1_OFF output_high(PIN_C3)
#define LED2_ON output_low(PIN_C4)
#define LED2_OFF output_high(PIN_C4)
#define LED3_ON output_low(PIN_C5)
#define LED3_OFF output_high(PIN_C5)
#define RELAY_ON output_high(PIN_A5)
#define RELAY_OFF output_low(PIN_A5)
#define vox_low 0x1E
//#define vox_mid 0x2D
#define vox_mid 0x5B
#define vox_hig 0x99
//
#byte OSCCON = 0x08F
#byte WPUA = 0x20C
#byte WPUC = 0x20E
#byte OPTION_REG = 0x095
#byte PCON = 0x10F
#byte OPA1CON = 0x096
#byte ANSELC = 0x187
//
// variables
int value_time = 0;
int value_sound = 0;
long count1 = 0;    // 0 to 65k
int relay_flag = 0;
//
// prototyping
#separate
void initializing();
#separate
void get_time();
#separate
void get_sound();
#separate
void drive_led();
//
// main function
void main()
{
   initializing();
   // main loop
   while(1)
   {
        //
        get_sound();
        drive_led();
        //
        if(value_sound > vox_mid)   // around 1.8V
        {
          RELAY_ON;
          get_time();
        }
        if(count1 > 0) RELAY_ON;
        else
        {
            if(input(PIN_A5) == 1)
            {
                setup_opamp1(OPAMP_DISABLED);
                RELAY_OFF;
                delay_ms(100);
                setup_opamp1(OPAMP_ENABLED);
            }
        }
   }
}
//
#separate
void initializing()
{
   setup_oscillator(OSC_4MHZ);
   //
   set_tris_a(Amode);   //
   set_tris_c(Cmode);   //
   //
   setup_comparator(NC_NC_NC_NC);
   // A/D converter initialize
   setup_adc(ADC_CLOCK_DIV_8);   // 2usec
   setup_adc_ports(sAN0 | VSS_VDD);    // PIN_A0
   //
   setup_opamp1(OPAMP_ENABLED | OPAMP_I_TO_OPAIN | OPAMP_NI_TO_OPAIN);
   // timer 1 intialize for hysteresis count down
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8); //8us
   set_timer1(0xE797);              // timer 1 setting almost 50msec
   enable_interrupts(INT_TIMER1);   // timer 1 enable
   enable_interrupts(GLOBAL);       // global enable
    //
    RELAY_OFF;
    LED1_OFF;
    LED2_OFF;
    LED3_OFF;
    //
   delay_ms(100);
}
// get sond from microphone
#separate
void get_sound()
{
   set_adc_channel(1);   //must wait 65u
   delay_us(100);
   value_sound = read_adc();   //0 to 255
   if(value_sound<2) value_sound = 2;   // 2 to 255
}
//
#separate
void drive_led()
{
    if(value_sound > vox_low) LED1_ON;
    else
    {
        LED1_OFF;
        LED2_OFF;
        LED3_OFF;
    }
    if(value_sound > vox_mid) LED2_ON;
    else
    {
        LED2_OFF;
        LED3_OFF;
    }
    if(value_sound > vox_hig) LED3_ON;
    else LED3_OFF;
}
// get time to turn on
#separate
void get_time()
{
    disable_interrupts(GLOBAL);       // global enable
    float count1f;
    set_adc_channel(0);   //must wait 65u
    delay_us(100);
    value_time = read_adc();   //0 to 255
    //if(value_time<3) value_time = 3;   // 3 to 240
    if(value_time>240) value_time = 240;   // 0 to 240
    count1 = value_time;
    count1f = count1;
    count1f = count1f / 3.43;   //0 to 70
    count1f = count1f * count1f;    //0 to 4900, center 68sec, max 245sec
    count1 = count1f;    //0 to 4900, center 68sec, max 245sec
    //count1 = value_time;    // 0 to 240
    //count1 = count1 * 20;    // 0 to 4800 *50msec
    if(count1 == 0) count1 = 3; // min 150ms
    enable_interrupts(GLOBAL);       // global enable
}
// timer 1 inturruption
#int_timer1
void isr1(void)
{
   set_timer1(0xE797);             // timer 1 almost 50msec
   if(count1 > 1) count1 = count1 - 1;
   else count1 = 0;
}
//

